#include <io.h>
#define pl r2	// Phase-Low
#define pm r3	// Phase-Mid
		// Phase-High = ZL
#define fl r4	// Frequenz-Low
#define fm r5	// Frequenz-Mid
#define fh r6	// Frequenz-High

/*
Kommandokodes, Binärkommandos < 0x20, ASCII >= 0x20 für Terminal
 00	nichts, kann man bspw. nach Synchronisationsverlust senden
 01	Frequenz setzen, danach 3 Bytes LSBfirst, kein Phasensprung
 02	Phase verschieben, danach 2 Bytes LSBfirst, kein Phasensprung
 03	Anwenderkurve setzen, danach 256 (vzl.) Bytes
 04	Frequenz abfragen (3 Bytes)
 05	Phase abfragen (2 Bytes) - natürlich nur bei geringen Frequenzen sinnvoll
 06	Anwenderkurve abfragen (256 Bytes)
 07	F_CPU abfragen (3 Bytes)
 '0'	Stopp, 0x80 ausgeben
 '1'	Sinuskurve
 '2'	Dreieck
 '3'	Sägezahn
 '4'	Rechteck
 '5'	Anwender
 '6'	Rauschen (nicht flash-basiert, ohne Frequenz und Phase, TODO)
 '+'	Frequenz höher 1 Schritt (geht nicht ohne Phasensprung auf 3 Bytes)
 '-'	Frequenz tiefer 1 Schritt
 'u'	Frequenz höher 256 Schritt (realisierbar ohne Phasensprung)
 'd'	Frequenz tiefer 256 Schritt
 'U'	Frequenz höher 65536 Schritt
 'D'	Frequenz tiefer 65536 Schritt
 'f'	Frequenzwert abfragen (ASCII)
 'k'	Kurve abfragen
 '?'	Echo
 */

main:
	clr	r1
	ldi	r18,1
	ldi	r19,2
phasenull:
	clr	pl
	clr	bm
	clr	ZL
	rjmp	mainloop

// gibt 1 Sample in 7 Takten aus
.macro NEXTSAMPLE
	add	pl,fl		// 1
	adc	pm,fm		// 1
	adc	ZL,fh		// 1
	lpm			// 3
	out	PORTB,r0	// 1, gesamt 7 Takte
.endm

// Liest 2 Bytes (R9:R8) von serieller Schnittstelle, Samples in 9 Takten ausgebend
.macro TWOBYTES
0:	NEXTSAMPLE
	in	r7,UCSRA
	bst	r7,7		// Nur das T-Flag überlebt NEXTSAMPLE
	NEXTSAMPLE
	brtc	0b
	in	r8,UDR
1:	NEXTSAMPLE
	in	r7,UCSRA
	bst	r7,7
	NEXTSAMPLE
	brtc	1b
	in	r9,UDR
.endm

// dito für 3 Bytes (R10:R9:R8)
.macro THREEBYTES
	TWOBYTES
3:	NEXTSAMPLE
	in	r7,UCSRA
	bst	r7,7
	NEXTSAMPLE
	brtc	3b
	in	r10,UDR
.endm

// Frequenz setzen
setF:	THREEBYTES
	NEXTSAMPLE
	movw	fl,r8
	mov	fh,r10
	NEXTSAMPLE
	rjmp	mainloop
	
// Phase verschieben (LSB bleibt unbeeinflusst)
addP:	TWOBYTES
	NEXTSAMPLE
	add	pm,r8
	adc	ZL,r9

mainloop:
	NEXTSAMPLE
	in	r7,UCSRA	// Daten von serieller Schnittstelle?
	bst	r7,7
	NEXTSAMPLE
	brtc	mainloop	// nein, in Hauptschleife verbleiben
	in	r24,UDR
	NEXTSAMPLE
	cpse	r24,r1		// ==0?
	 clt
	NEXTSAMPLE
	brts	mainloop	// Nichts tun
	set
	NEXTSAMPLE
	cpse	r24,r18		// ==1?
	 clt
	NEXTSAMPLE
	brts	setF		// Frequenz setzen
	set
	NEXTSAMPLE
	cpse	r24,r19		// ==2?
	 cld
	NEXTSAMPLE
	brts	addP		// Phase schieben
// ab hier Verarbeitung der "unkritischen" Kommandos,
// also solche, die die kontinuierliche Ausgabe unterbrechen
	cpi	r24,2
	breq	blabla

// Phase nullsetzen (springt sowieso, also warum nicht definierten
// Ausgangszustand schaffen?)
	rjmp	phasenull

noise25:	// aus xapp052.pdf
// Bit 0 := Bit 24 XOR Bit 21 -> 9 Takte! Genauso schnell wie DDS!
	rol	r2	// vorhergehendes CY = Bit 34 einschieben
	rol	r3
	rol	r19
	bst	r19,6	// war Bit 21, nach dem Schieben Bit 22
	bld	r18,0
	eor	r2,r18	// verändert CY nicht (nett!)
	out	PORTB,r19
	rjmp	noise25	// hier: Unterbrechung nur per ISR; 9 Takte
